#include <pspkernel.h>
#include <pspdebug.h>
#include <pspaudio.h>
#include <pspaudiolib.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define NUM_BUFFERS 2
#define SAMPLESIZE 1024

typedef struct _priv_data {
	int num_ch;
	int	channel;
	u8	*rawbuf;
	u8	*mixbufs[NUM_BUFFERS];
	int		next_buffer;
	void (*callback)(u8 *stream, int len);
} priv_data;

static priv_data psp_audio;
static int buf_size;
static int mixlen;
static int ao_thid;
static int audio_abort;
static int audio_paused;

static u8 *get_audio_buf()
{
	return psp_audio.mixbufs[psp_audio.next_buffer];
}

static void play_audio()
{
	u8 *mixbuf = psp_audio.mixbufs[psp_audio.next_buffer];

	if (psp_audio.num_ch == 1)
		sceAudioOutputBlocking(psp_audio.channel, PSP_AUDIO_VOLUME_MAX, mixbuf);
	else
		sceAudioOutputPannedBlocking(psp_audio.channel, PSP_AUDIO_VOLUME_MAX, PSP_AUDIO_VOLUME_MAX, mixbuf);

	psp_audio.next_buffer = (psp_audio.next_buffer + 1) % NUM_BUFFERS;
}

static int audio_thread(SceSize args, void *argp)
{
	u8 *stream;

	while(!audio_abort)
	{
		stream = get_audio_buf();
		memset(stream, 0x00, buf_size);
		if (!audio_paused)
		{
			psp_audio.callback(stream, buf_size);
		}
		play_audio();
	}
	return 0;
}

int init_audio(int ch, void (*audio_callback)(u8 *stream, int len))
{
	int i;

	if (ch <= 0 || ch > 2)
		return 0;
//	buf_size = SAMPLESIZE*2*2;
//	mixlen = SAMPLESIZE*2*2*NUM_BUFFERS;
	buf_size = SAMPLESIZE*ch*2;
	mixlen = SAMPLESIZE*ch*2*NUM_BUFFERS;
	psp_audio.rawbuf = (u8 *) memalign(64, mixlen);
	psp_audio.num_ch = ch;
	if (psp_audio.rawbuf == NULL)
		return 0;
	if (ch > 1)
		psp_audio.channel = sceAudioChReserve(PSP_AUDIO_NEXT_CHANNEL, SAMPLESIZE, PSP_AUDIO_FORMAT_STEREO);
	else
		psp_audio.channel = sceAudioChReserve(PSP_AUDIO_NEXT_CHANNEL, SAMPLESIZE, PSP_AUDIO_FORMAT_MONO);
	if (psp_audio.channel < 0) {
		free(psp_audio.rawbuf);
		psp_audio.rawbuf = NULL;
		return 0;
	}
	memset(psp_audio.rawbuf, 0, mixlen);
	for (i = 0; i < NUM_BUFFERS; i++) {
		psp_audio.mixbufs[i] = &psp_audio.rawbuf[i * buf_size];
	}

	psp_audio.next_buffer = 0;
	psp_audio.callback = audio_callback;
	ao_thid = sceKernelCreateThread ("audio thread", (SceKernelThreadEntry)audio_thread, 0x12, 0x50000, THREAD_ATTR_VFPU, 0);
//	ao_thid = sceKernelCreateThread ("audio thread", (SceKernelThreadEntry)audio_thread, 0x13, 0x50000, THREAD_ATTR_VFPU, 0);
	if (ao_thid < 0)
	{
		free(psp_audio.rawbuf);
		psp_audio.rawbuf = NULL;
		return 0;
	}
	audio_abort = 0;
	audio_paused = 0;
	sceKernelStartThread(ao_thid, 0, 0);
	return 1;
}

void close_audio()
{
	audio_abort = 1;
	sceKernelWaitThreadEnd(ao_thid, NULL);
	sceKernelDeleteThread(ao_thid);
	if (psp_audio.channel >= 0) {
		sceAudioChRelease(psp_audio.channel);
		psp_audio.channel = -1;
	}
	free(psp_audio.rawbuf);
	psp_audio.rawbuf = NULL;
}

/* toggle */
void audio_pause()
{
	audio_paused = audio_paused ? 0 : 1;
}
