#include <pspkernel.h>
#include <pspdebug.h>
#include <pspthreadman.h>
#include <pspctrl.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mytimer.h"
#include "myevent.h"

static mytimer my_timers[MAX_TIMER];
static myevent_q event_q;
SceUID timer_sema;
SceUID event_sema;
SceUID event_thread;
int event_handler_close = 0;

static void init_timer(void);
static void close_timer(void);

static double gettime_usec()
{    
    struct timeval tv;
    gettimeofday(&tv,NULL);
    return (double)tv.tv_sec * 1000000 + tv.tv_usec;
}

void add_event(int event, void *userdata)
{
	sceKernelWaitSema(event_sema, 1, 0);
	if( ( event_q.head+3 ) % MAX_EVENT == event_q.tail ) {
		/* do nothing */
	}

	if( ( event_q.head+1 ) % MAX_EVENT != event_q.tail )
	{
		event_q.head = ( event_q.head+1 ) % MAX_EVENT;
		event_q.event[ event_q.head ].value = event;
		event_q.event[ event_q.head].userdata = userdata;
	}
	sceKernelSignalSema(event_sema, 1);
}

int get_event(myevent *event)
{
	int fail = 0;
	sceKernelWaitSema(event_sema, 1, 0);
    if ( event_q.head != event_q.tail ) {
      event_q.tail = ( event_q.tail+1 ) % MAX_EVENT;
      event->value = event_q.event[event_q.tail].value;
	  event->userdata = event_q.event[event_q.tail].userdata;
    }
	else
	{
		event->value = 0;
		event->userdata = NULL;
		fail = 1;
	}
	sceKernelSignalSema(event_sema, 1);
	if (fail)
		return 0;
	else
		return 1;
}

static int event_handler(SceSize args, void *argp)
{
	static double last_check_time, last_key_check_time;
	SceCtrlData pad;
	int key_repeating = 0;
	static int old_key = 0;
	int i;

	event_sema = sceKernelCreateSema("", 0, 1, 1, 0);
	sceKernelWaitSema(event_sema, 1, 0);
	memset((void *)&event_q, 0, sizeof(event_q));
	sceKernelSignalSema(event_sema, 1);
	last_key_check_time = last_check_time = gettime_usec();
	sceCtrlSetSamplingCycle(0);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
	init_timer();
	for (;;)
	{
		if (event_handler_close)
			break;
		if ((gettime_usec() - last_check_time) > 1000)
		{
			sceCtrlReadBufferPositive(&pad, 1);
			if (pad.Buttons == old_key && old_key != 0)
			{
				int interval;
				if (key_repeating)
				{
					interval = 100000;
				}
				else
				{
					interval = 1000000;
				}
				if ((gettime_usec() - last_key_check_time) > interval)
				{
					key_repeating = 1;
					add_event(pad.Buttons, NULL);
					old_key = pad.Buttons;
					last_key_check_time = gettime_usec();
				}
			}
			else
			{
				key_repeating = 0;
				if (pad.Buttons != 0)
					add_event(pad.Buttons, NULL);
				old_key = pad.Buttons;
				last_key_check_time = gettime_usec();
			}
			last_check_time = gettime_usec();
		}
		/* now check the timers */
		for (i = 0; i < MAX_TIMER; i++)
		{
			sceKernelWaitSema(timer_sema, 1, 0);
			if (my_timers[i].id >= 0)
			{
				if ((gettime_usec() - my_timers[i].last_time) > my_timers[i].interval*1000)
				{
					if (my_timers[i].callback)
					{
//						my_timers[i].callback();
						my_timers[i].callback(my_timers[i].user_data);
					}
					if (!my_timers[i].repeat)
					{
						my_timers[i].id = -1;
						my_timers[i].callback = NULL;
					}
					else
					{
						my_timers[i].last_time = gettime_usec();
					}
				}
			}
			sceKernelSignalSema(timer_sema, 1);
		}
		sceKernelDelayThread(10);
	}
	sceKernelDeleteSema(event_sema);
	close_timer();
	return 0;
}

static void init_timer(void)
{
	int i;
	timer_sema = sceKernelCreateSema("Timer", 0, 1, 1, 0);
	sceKernelWaitSema(timer_sema, 1, 0);
	memset((void *)my_timers, 0, sizeof(my_timers));
	for (i = 0; i < MAX_TIMER; i++)
	{
		my_timers[i].id = -1;
	}
	sceKernelSignalSema(timer_sema, 1);
}

static void close_timer(void)
{
	sceKernelDeleteSema(timer_sema);
}

void init_event_handler()
{
//	event_thread = sceKernelCreateThread("Event", event_handler, 0x08, 0x5000, 0, NULL);
	event_thread = sceKernelCreateThread("Event", event_handler, 0x11, 0x5000, 0, NULL);
	if (event_thread >= 0)
	{
		sceKernelStartThread(event_thread, 0, 0);
	}
}

void close_event_handler()
{
	event_handler_close = 1;
	sceKernelWaitThreadEnd(event_thread, NULL);
	sceKernelDeleteThread(event_thread);
}

static int get_free_slot()
{
	int i;
	int free_slot = -1;
	sceKernelWaitSema(timer_sema, 1, 0);
	for (i = 0; i < MAX_TIMER; i++)
	{
		if (my_timers[i].id == -1)
		{
			/* its free */
			break;
		}
	}
	if (i < MAX_TIMER)
	{
		free_slot = i;
	}
	sceKernelSignalSema(timer_sema, 1);
	return free_slot;
}

/* -1 means error */
int set_timer(int interval, void (*callback)(void *priv), void *user_data, int repeat)
{
	int id;
	if (interval == 0)
		return -1;
	id = get_free_slot();
	if (id < 0)
		return -1;
	sceKernelWaitSema(timer_sema, 1, 0);
	my_timers[id].id = id;
	my_timers[id].interval = interval;
	my_timers[id].repeat = repeat;
	my_timers[id].callback = callback;
	my_timers[id].user_data = user_data;
	my_timers[id].last_time = gettime_usec();
	sceKernelSignalSema(timer_sema, 1);
	return id;
}

int reset_timer(int id)
{
	if (id < 0)
	{
		return -1;
	}
	sceKernelWaitSema(timer_sema, 1, 0);
	memset((void *)&my_timers[id], 0, sizeof(mytimer));
	sceKernelSignalSema(timer_sema, 1);
	return 0;
}
