#include <pspkernel.h>
#include <pspdebug.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <png.h>

#include "myscreen.h"
#include "mycolor.h"
#include "mydrawbmp.h"
#include "myscrolbar.h"
#include "mytext.h"
#include "mystring.h"

void free_png_img(my_bmp_type *img)
{
  if (img)
  {
    free(img->data);
	free(img);
  }
}

void user_warning_fn(png_structp png_ptr, png_const_charp warning_msg)
{
	// ignore PNG warnings
}

/* Load an image and show it to screen */
my_bmp_type* read_png_file (char * filename)
{
//	u32* vram32;
//	u16* vram16;
//	int bufferwidth;
//	int pixelformat;
//	int unknown;
	png_structp png_ptr;
	png_infop info_ptr;
	unsigned int sig_read = 0;
	png_uint_32 width, height;
	int bit_depth, color_type, interlace_type, x, y;
	u32* line;
	FILE *fp;
	my_bmp_type *img = NULL;

	if ((fp = fopen(filename, "rb")) == NULL)
		return NULL;
	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if (png_ptr == NULL) {
		fclose(fp);
		return NULL;
	}
	png_set_error_fn(png_ptr, (png_voidp) NULL, (png_error_ptr) NULL, user_warning_fn);
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL) {
		fclose(fp);
		png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
		return NULL;
	}
	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, sig_read);
	png_read_info(png_ptr, info_ptr);
	png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, int_p_NULL, int_p_NULL);
	png_set_strip_16(png_ptr);
	png_set_packing(png_ptr);
	if (color_type == PNG_COLOR_TYPE_PALETTE) png_set_palette_to_rgb(png_ptr);
	if (color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8) png_set_gray_1_2_4_to_8(png_ptr);
	if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) png_set_tRNS_to_alpha(png_ptr);
	png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);
	line = (u32*) malloc(width * 4);
	if (!line) {
		fclose(fp);
		png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
		return NULL;
	}
	img = (my_bmp_type *)malloc(sizeof(my_bmp_type));
	if (!img)
	{
		free(line);
		fclose(fp);
		png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
		return NULL;
	}
	img->data = (u32 *)malloc(width * height * 4);
	if (!img->data)
	{
		free(line);
		free(img);
		fclose(fp);
		png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
		return NULL;
	}
    img->current_frame = 0;
	img->frame = 1;
	img->w = width;
	img->h = width;

	for (y = 0; y < height; y++) {
		png_read_row(png_ptr, (u8*) line, png_bytep_NULL);
		for (x = 0; x < width; x++) {
			u32 color32 = line[x];

//			int r = color32 & 0xff;
//			int g = (color32 >> 8) & 0xff;
//			int b = (color32 >> 16) & 0xff;
			img->data[x + y * img->w] = color32;
		}
	}
	free(line);
	png_read_end(png_ptr, info_ptr);
	png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
	fclose(fp);
	return img;
}
