
/* 
// PSP 
Malloc(x) & Free() Yoshihiro 

*/ 


#include <stdlib.h> 
#include <errno.h> 

#include <sys/types.h> 
#include <unistd.h> 
#include <stddef.h> 
#include <stdarg.h> 

#include "avcodec.h"
#include "bget.h"

/* here we can use OS dependant allocation functions */
#undef malloc
#undef free
#undef realloc

int has_initialized = 0; 

#define MAX_AV_HEAP_SIZE 6*1024*1024

char __attribute__((aligned(16))) av_heap[MAX_AV_HEAP_SIZE];

void *malloc_psp(unsigned int size)
{
	if (!has_initialized)
	{
		bpool(av_heap, MAX_AV_HEAP_SIZE);
		has_initialized = 1;
	}
	return bget(size);
}

void *realloc_psp(void *ptr, unsigned int size)
{
	return bgetr(ptr, size);
}

void free_psp(void *ptr)
{
	brel(ptr);
}

void *av_malloc(unsigned int size)
{
    void *ptr;

    ptr = malloc_psp(size);
    return ptr;
}

/**
 * av_realloc semantics (same as glibc): if ptr is NULL and size > 0,
 * identical to malloc(size). If size is zero, it is identical to
 * free(ptr) and NULL is returned.  
 */
void *av_realloc(void *ptr, unsigned int size)
{
    return realloc_psp(ptr, size);
}

/* NOTE: ptr = NULL is explicetly allowed */
void av_free(void *ptr)
{
    /* XXX: this test should not be needed on most libcs */
    if (ptr)
        free_psp(ptr);
}
